;---------------------------------------------------------------------------------------------;
;
; example 3b (modified example 3)
; 68681 board - duart setup and serial transmission
;
; this version runs on systems with 4MB ROM, 8MB RAM, & DUART @ $AA0000
;
; 17/11/2020 steve crompton
;
;---------------------------------------------------------------------------------------------;

ram_start           EQU $000000
rom_start           EQU $C00000

board_68681         EQU $AA0000
board_hex_display   EQU $AB0000

cr                  EQU 13          ;carriage return
lf                  EQU 10          ;line feed

duart_mr1a          EQU 0*2         ;mode register a
duart_mr2a          EQU 0*2         ;mode register a
duart_sra           EQU 1*2         ;status register a
duart_csra          EQU 1*2         ;clock-select register a
duart_cra           EQU 2*2         ;command register a
duart_rba           EQU 3*2         ;receiver buffer a
duart_tba           EQU 3*2         ;transmitter buffer a
duart_ipcr          EQU 4*2         ;input port change register
duart_acr           EQU 4*2         ;auxiliary control register
duart_isr           EQU 5*2         ;interrupt status register
duart_imr           EQU 5*2         ;interrupt mask register
duart_cur           EQU 6*2         ;counter mode: current msb of counter
duart_ctur          EQU 6*2         ;counter/timer upper register
duart_clr           EQU 7*2         ;counter mode: current lsb of counter
duart_ctlr          EQU 7*2         ;counter/timer lower register
duart_mr1b          EQU 8*2         ;mode register b
duart_mr2b          EQU 8*2         ;mode register b
duart_srb           EQU 9*2         ;status register b
duart_csrb          EQU 9*2         ;clock-select register b
duart_crb           EQU 10*2        ;command register b
duart_rbb           EQU 11*2        ;receiver buffer b
duart_tbb           EQU 11*2        ;transmitter buffer b
duart_ivr           EQU 12*2        ;interrupt-vector register
duart_opcr          EQU 13*2        ;output port configuration register
duart_start_counter EQU 14*2        ;start-counter command
duart_opr_set       EQU 14*2        ;bit set command
duart_stop_counter  EQU 15*2        ;stop-counter command
duart_opr_reset     EQU 15*2        ;bit reset command

duart_rxrdy         EQU 0           ;receiver ready
duart_txrdy         EQU 2           ;transmitter ready

lword_display       EQU board_hex_display
duart_base_addr     EQU board_68681+1

    ORG     rom_start

    DC.L    ram_start+$1000         ;reset: initial ssp
    DC.L    cold_boot               ;reset: initial pc

code:

    ORG     rom_start+$400

cold_boot:

    move.l #$12340000,d0            ;put "12340000" on the hex display
    move.l d0,lword_display         ;to indicate program start

    lea.l  duart_base_addr,a0       ;load the address of the duart into a0

    jsr duart_init                  ;reset/initialise the duart

    move.b #cr,d0                   ;send carriage return
    jsr duart_put
    move.b #'H',d0                  ;send 'H'
    jsr duart_put
    move.b #'e',d0                  ;send 'e'
    jsr duart_put
    move.b #'l',d0                  ;send 'l'
    jsr duart_put
    move.b #'l',d0                  ;send 'l'
    jsr duart_put
    move.b #'o',d0                  ;send 'o'
    jsr duart_put
    move.b #' ',d0                  ;send ' '
    jsr duart_put
    move.b #'W',d0                  ;send 'W'
    jsr duart_put
    move.b #'o',d0                  ;send 'o'
    jsr duart_put
    move.b #'r',d0                  ;send 'r'
    jsr duart_put
    move.b #'l',d0                  ;send 'l'
    jsr duart_put
    move.b #'d',d0                  ;send 'd'
    jsr duart_put
    move.b #cr,d0                   ;send carriage return
    jsr duart_put
    move.b #lf,d0                   ;send line feed
    jsr duart_put

    move.l #$00005678,d0            ;put "00005678" on the hex display
    move.l d0,lword_display         ;to indicate program end

forever:

    bra forever                     ;wait for reset button to be pressed

;---------------------------------------------------------------------------------------------;

; subroutine:   duart_init

; description:  68681 duart - reset/initialise device routine
; purpose:      prepare device for use
; parameters:   a0.l - base address of the 68681 duart
; side effects:

; handshaking:  rx-rts (op0 pin 26) [output low - channel a rx ready ]
;               tx-cts (ip0 pin 7)  [input  low - channel a tx enable]

duart_init:

;reset duart channel a

    move.b #$0a,duart_cra(a0)       ;disable transmitter & receiver
    move.b #$20,duart_cra(a0)       ;reset receiver
    move.b #$30,duart_cra(a0)       ;reset transmitter
    move.b #$40,duart_cra(a0)       ;reset error status
    move.b #$50,duart_cra(a0)       ;reset channel's break-change interrupt
    move.b #$10,duart_cra(a0)       ;reset mr pointer to mr1

;configure duart channel a
;38400 baud, 8 data bits, no parity, 1 stop bit, + hardware handshaking

    move.b #$00,duart_acr(a0)       ;select baud rates - use set 1
    move.b #$cc,duart_csra(a0)      ;set rx-baud to 38400, set tx-baud to 38400
    move.b #$93,duart_mr1a(a0)      ;enable rx-rts, set no parity, set 8 data bits
    move.b #$17,duart_mr2a(a0)      ;enable tx-cts, set 1 stop bit
    move.b #$05,duart_cra(a0)       ;enable transmitter & receiver
    move.b #$01,duart_opr_set(a0)   ;set rx-rts pin low
    rts

;---------------------------------------------------------------------------------------------;

; subroutine:   duart_put

; description:  68681 duart - serial transmit routine
; purpose:      transmit a byte/character
; parameters:   a0.l - base address of the 68681 duart
;               d0.b - byte/character to send
; side effects:

duart_put:

;wait for transmitter ready

    btst.b #duart_txrdy,duart_sra(a0)   ;read the status register
    beq.s duart_put                     ;txrdy=1 when transmitter ready

    move.b d0,duart_tba(a0)             ;transmit the byte/character in d0
    rts

;---------------------------------------------------------------------------------------------;

    END    code






















































*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
